<?php

namespace Drupal\commerce_mollie\Controller;

use Drupal\commerce_mollie\ErrorHelper;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Validates payments that are posted by Mollie on the webhook-endpoint.
 *
 * @package Drupal\commerce_mollie\Controller
 */
class Webhook extends ControllerBase {

  /**
   * The post endpoint.
   *
   * @return JsonResponse
   *   Empty JsonResponse whatever has been posted.
   */
  public function post() {

    $mollie_payment_id = \Drupal::request()->request->get('id');
    if (!$mollie_payment_id) {
      return new JsonResponse();
    }

    /*
     * Load CommercePayment.
     */
    $entityManager = \Drupal::service('entity.manager');

    $paymentIds = $entityManager->getStorage('commerce_payment')
      ->getQuery()->condition('remote_id', $mollie_payment_id)
      ->execute();
    $paymentItems = $entityManager->getStorage('commerce_payment')
      ->loadMultiple($paymentIds);

    if (!$paymentItems) {
      return new JsonResponse();
    }

    /** @var \Drupal\commerce_payment\Entity\Payment $payment */
    $payment = reset($paymentItems);

    // Only process payments that are authorization state.
    if ($payment->getState()->value !== 'authorization') {
      return new JsonResponse();
    }
    if (empty($payment->getOrder())) {
      return new JsonResponse();
    }

    /** @var \Drupal\commerce_mollie\Plugin\Commerce\PaymentGateway\Mollie $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    /** @var \Mollie_API_Client $api */
    $api = $payment_gateway_plugin->getApi();

    /*
     * Handle status that Mollie returns.
     */
    $mollie_payment = $api->payments->get($mollie_payment_id);
    try {

      // Payment has been received on Mollie's side.
      if ($mollie_payment->isPaid() === TRUE) {

        // Apply payment 'capture' transition.
        $capture_transition = $payment->getState()
          ->getWorkflow()
          ->getTransition('capture');
        $payment->getState()->applyTransition($capture_transition);
        $payment->setCapturedTime(REQUEST_TIME);
        $payment->save();

        // Apply order 'validate' transition. Changes the state to Completed.
        $order = $payment->getOrder();
        $validate_transition = $order->getState()
          ->getWorkflow()
          ->getTransition('validate');
        $order->getState()->applyTransition($validate_transition);
        $order->save();

      }

      // Payment was not succesful.
      // @todo handle multiple statuses like 'cancel'.
      elseif ($mollie_payment->isOpen() === FALSE) {

        // Apply payment 'void' transition.
        $capture_transition = $payment->getState()
          ->getWorkflow()
          ->getTransition('void');
        $payment->getState()->applyTransition($capture_transition);
        $payment->setCapturedTime(REQUEST_TIME);
        $payment->save();

        // Apply order 'cancel' transition.
        $order = $payment->getOrder();
        $cancel_transition = $order->getState()
          ->getWorkflow()
          ->getTransition('cancel');
        $order->getState()->applyTransition($cancel_transition);
        $order->save();


      }

    }

    catch (\Mollie_API_Exception $e) {
      ErrorHelper::handleException($e);
    }

    return new JsonResponse();
  }

}
