<?php

namespace Drupal\commerce_mollie\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_mollie\ErrorHelper;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the Mollie payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "mollie",
 *   label = "Mollie",
 *   display_label = "Mollie",
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_mollie\PluginForm\OffsiteRedirect\MolliePaymentOffsiteForm",
 *   },
 *   payment_method_types = {"mollie"},
 * )
 */
class Mollie extends OffsitePaymentGatewayBase {

  /**
   * The Mollie gateway used for making API calls.
   *
   * @var \Mollie_API_Client
   */
  protected $api;

  /**
   * The Mollie paymentUrl for a specific Payment/Transaction.
   */
  protected $molliePaymentUrl = '';

  /**
   * This is the return-url on Drupal's side where Mollie redirects back to.
   */
  protected $redirectUrl = '';

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager);

    // MollieAPIClient should be setup in the payment_method?
    $this->api = new \Mollie_API_Client();

    // Initialize api_key.
    try {
      if (!empty($configuration['api_key_live']) && $configuration['mode'] === 'live') {
        $this->api->setApiKey($configuration['api_key_live']);
      }
      elseif (!empty($configuration['api_key_test']) && $configuration['mode'] === 'test') {
        $this->api->setApiKey($configuration['api_key_test']);
      }
    }
    catch (\Mollie_API_Exception $e) {
      ErrorHelper::handleException($e);
    }

  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {

    return [
        'redirect_method' => 'post',
        'api_key_test' => 'test_',
        'api_key_live' => 'live_',
        'callback_domain' => '',
      ] + parent::defaultConfiguration();

  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $host = \Drupal::request()->getHost();

    $form['api_key_test'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Mollie Test API-key'),
      '#default_value' => $this->configuration['api_key_test'],
      '#required' => TRUE,
    ];
    $form['api_key_live'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Mollie Live API-key'),
      '#default_value' => $this->configuration['api_key_live'],
      '#required' => TRUE,
    ];
    $form['callback_domain'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Callback domain'),
      '#description' => $this->t('Domain that Mollie should use to post transaction results to.<br/>
      <ul>
        <li> Do not add an "/" after .com</li>
        <li>You must include https:// or http://</li>
      </ul>
      For live websites this is the your live domain</br><br/>
      
      By default (empty) the domain visible in the browser window will be used.<br/>
      Currently that is: @host<br/><br/>
      
      While developing you can use a tunneled domain that hits your local development machine.<br/>
      This can be done with https://localtunnel.github.io/www<br/>
      <code>lt --port 80 --local-host @host</code>
      ', ['@host' => $host]),
      '#default_value' => $this->configuration['callback_domain'],
      '#required' => FALSE,
    ];

    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['api_key_test'] = $values['api_key_test'];
      $this->configuration['api_key_live'] = $values['api_key_live'];
      $this->configuration['callback_domain'] = $values['callback_domain'];
    }

  }

  /**
   * Return initalized Mollie API.
   */
  public function getApi() {
    return $this->api;
  }

  /**
   * {@inheritdoc}
   */
  public function getNotifyUrl() {

    // Get callback_domain from configuration or use reqeust root.
    if (!$callback_domain = $this->configuration['callback_domain']) {
      global $base_root;
      $callback_domain = $base_root;
    };

    // @see http://drupal.stackexchange.com/questions/187086/trustedresponseredirect-failing-how-to-prevent-cache-metadata
    $webhook_endpoint = Url::fromRoute('commerce_mollie.webhook_post')
      ->toString(TRUE)->getGeneratedUrl();

    return Url::fromUri($callback_domain . $webhook_endpoint)->setAbsolute();
  }

  /**
   * Setter $molliePaymentUrl.
   *
   * This is end-point on Mollie's side.
   */
  public function setMolliePaymentUrl($molliePaymentUrl = '') {
    $this->molliePaymentUrl = $molliePaymentUrl;
    return $molliePaymentUrl;
  }

  /**
   * Getter $molliePaymentUrl.
   *
   * This is end-point on Mollie's side.
   */
  public function getMolliePaymentUrl() {
    return $this->molliePaymentUrl;
  }

  /**
   * Setter $redirectUrl.
   *
   * This is the return-url on Drupal's side where Mollie redirects back to.
   *
   * @todo Use this different for MolliePaymentOffsiteForm and MolliePaymentUrl.
   */
  public function setRedirectUrl($redirectUrl) {
    $this->redirectUrl = $redirectUrl;
  }

  /**
   * Getter $redirectUrl.
   *
   * This is the return-url on Drupal's side where Mollie redirects back to.
   *
   * @todo Use this different for MolliePaymentOffsiteForm and MolliePaymentUrl.
   */
  public function getRedirectUrl() {

    return $this->redirectUrl;

  }

  /**
   * {@inheritdoc}
   *
   * @todo resolve and use this in controller/RedirectUrl::get
   */
  public function onReturn(OrderInterface $order, Request $request) {

  }

  /**
   * {@inheritdoc}
   *
   * @todo resolve and use this in controller/RedirectUrl::get
   */
  public function onCancel(OrderInterface $order, Request $request) {
    drupal_set_message($this->t('You have canceled checkout at @gateway but may resume the checkout process here when you are ready.', [
      '@gateway' => $this->getDisplayLabel(),
    ]));
  }

  /**
   * {@inheritdoc}
   *
   * @todo resolve this in controller/Webhook::post
   */
  public function onNotify(Request $request) {

  }

  /**
   * {@inheritdoc}
   */
  public function createRemotePayment(PaymentInterface $payment, $capture = TRUE) {

    /** @var \Drupal\commerce_mollie\Plugin\Commerce\PaymentGateway\Mollie $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();

    // If the redirect_url is not set, default to commerce_checkout.
    // @todo: move the default setting to the gateway constructor or defaultConfiguration()
    // @todo: use commerce_payment.pending route instead of complete, because Mollie has an a-sync callback (the webhook).
    // @see https://www.drupal.org/node/2864515
    // @see https://www.drupal.org/node/2856042
    if (!$payment_gateway_plugin->getRedirectUrl()) {
      $commerce_checkout_redirect_url = Url::fromRoute('commerce_checkout.form', ['commerce_order' => $payment->getOrderId()], ['absolute' => 'true'])->toString();
      $commerce_checkout_redirect_url = $commerce_checkout_redirect_url . '/payment/return';
      $payment_gateway_plugin->setRedirectUrl($commerce_checkout_redirect_url);
    }

    $transaction_data = [
      'amount' => $payment->getAmount()->getNumber(),
      'description' => $this->t('@store order @order_id', [
        '@store' => $payment->getOrder()->getStore()->label(),
        '@order_id' => $payment->getOrderId(),
      ]),
      'redirectUrl' => $payment_gateway_plugin->getRedirectUrl(),
      'webhookUrl' => $payment_gateway_plugin->getNotifyUrl()->toString(),
      'metadata' => [
        'order_id' => $payment->getOrderId(),
      ],
    ];

    // Create URL for Mollie payment.
    $api = $payment_gateway_plugin->getApi();
    try {
      $mollie_payment = $api->payments->create($transaction_data);
      $payment->setRemoteId($mollie_payment->id);
      $payment->setRemoteState($mollie_payment->status);

      // Change the step from 'new' to 'authorization'.
      $authorize_transition = $payment->getState()->getWorkflow()->getTransition('authorize');
      $payment->getState()->applyTransition($authorize_transition);
      $payment->save();

      // Change the step from 'draft' to 'validation'.
      $order = $payment->getOrder();
      $validate_transition = $order->getState()
        ->getWorkflow()
        ->getTransition('place');
      $order->getState()->applyTransition($validate_transition);
      $order->save();

      $redirect_url = $payment_gateway_plugin->setMolliePaymentUrl($mollie_payment->getPaymentUrl());

      ErrorHelper::handleErrors($mollie_payment);
      ErrorHelper::handleErrors($redirect_url);
    }
    catch (\Mollie_API_Exception $e) {
      ErrorHelper::handleException($e);
    }

  }

  /**
   * Capture payment and update order-state to 'validate' or 'place' depending
   * on checkout flow that is chosen.
   *
   * @todo use this method.
   */
  public function capturePayment(PaymentInterface $payment, Price $amount = NULL) {

  }

}
